package pps.core.common.utils;

import com.google.common.base.Strings;
import pps.core.common.config.MqConfig;
import pps.core.common.config.TraceLogConfig;
import pps.core.common.mq.CommonTraceLogMessage;
import xstartup.base.XContext;
import xstartup.base.XServiceContext;
import xstartup.base.mq.XMqFactory;
import xstartup.base.mq.XMqProduceQueue;
import xstartup.base.mq.XMqProducer;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XServiceResult;

import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.LongAdder;

public enum CommonTraceLogger {
    INSTANCE;

    private final LinkedBlockingQueue<TraceHelper.TraceBuilder> traceBuilderQueue =
            new LinkedBlockingQueue<TraceHelper.TraceBuilder>(10240);
    private final LongAdder exceptionCounter = new LongAdder();
    private final LongAdder insertedCounter = new LongAdder();
    private ConcurrentHashMap<String, String> traceIds = new ConcurrentHashMap<String, String>();
    private XMqProducer<CommonTraceLogMessage> producer = null;
    private XContext context;
    private final Thread workerThread = new Thread(new Runnable() {
        @Override
        public void run() {
            executeWork();
        }
    });

    public synchronized void start() {
        if (workerThread.getState().equals(Thread.State.NEW)) {
            workerThread.start();
        }
    }

    public long getExceptionCount() {
        return exceptionCounter.longValue();
    }

    public long getInsertCount() {
        return insertedCounter.longValue();
    }

    public long tasks() {
        return this.traceBuilderQueue.size();
    }


    public void add(XContext context) {
        TraceHelper.TraceBuilder traceBuilder = TraceHelper.getTraceBuilder(context);
        if (traceBuilder == null) {
            return;
        }

        add(traceBuilder, context);
    }

    public void add(TraceHelper.TraceBuilder builder) {
        this.add(builder, null);
    }


    public void add(TraceHelper.TraceBuilder builder, XContext context) {

        // 读取配置
        Boolean enableTraceLog = context == null ? true : TraceLogConfig.checkEnableTraceLog(context);// TraceLogConfig.enableTraceLog != null ? TraceLogConfig.enableTraceLog.getValue(context) : null;
        if (context != null && enableTraceLog != null && !enableTraceLog.booleanValue()) {
            return;
        }

        try {
            if (context != null) {
                context.getLogger().info(builder.buildTraceLog());
            }
            traceBuilderQueue.put(builder);
            CounterBuilder.globalCounterBuilder.addCounter("CommonTraceLog-Add");
        } catch (InterruptedException e) {
            exceptionCounter.increment();
        }
    }


    private void executeWork() {
        while (true) {
            try {
                if (context == null) {
                    this.context = XServiceContext.build(CommonTraceLogger.class);
                }
                if (this.producer == null) {
                    this.producer = this.createProducer(this.context);
                }
                if (this.producer == null) {
                    CounterBuilder.globalCounterBuilder.addCounter("CommonTraceLogger-producerError");
                    XThreadHelper.sleep(5000);
                    continue;
                }
                TraceHelper.TraceBuilder traceBuilder = traceBuilderQueue.take();
                CommonTraceLogMessage message = new CommonTraceLogMessage();
                message.setCreateTime(new Date());
                message.setTraceid(traceBuilder.getTraceId());
                message.setTraceInfo2(traceBuilder.buildTraceLog());
                XServiceResult publishResult = this.producer.publish(context, message);
                if (!publishResult.isSuccess()) {
                    CounterBuilder.globalCounterBuilder.addCounter("CommonTraceLog-Posted-Failed");
                } else {
                    CounterBuilder.globalCounterBuilder.addCounter("CommonTraceLog-Posted-Success");
                }
            } catch (Exception ex) {
                exceptionCounter.increment();
                CounterBuilder.globalCounterBuilder.addCounter(ex, this.getClass());
                CounterBuilder.globalCounterBuilder.addCounter("CommonTraceLog-WorkException");
                XThreadHelper.sleep(1000);
            }
        }
    }

    private XMqProducer<CommonTraceLogMessage> createProducer(XContext context) {
        String queueName = MqConfig.CommonTraceLogMq.getValue(context);
        if (Strings.isNullOrEmpty(queueName)) {
            throw new RuntimeException("CommonTraceLogMq主题配置错误");
        }
        XMqProduceQueue queue = new XMqProduceQueue.Builder().queue(queueName).build();
        XMqProducer<CommonTraceLogMessage> producer = XMqFactory.createProducer(context, queue, CommonTraceLogMessage.class);
        return producer;
    }


}
