package pps.core.common.utils;

import com.google.common.base.Strings;
import xstartup.annotation.XText;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

@XText("动态对象容器")
public class DynObj extends TreeMap<String, Object> implements Serializable {
    private static final long serialVersionUID = 1L;

    public DynObj() {
        super(String.CASE_INSENSITIVE_ORDER);
    }

    public DynObj(Map obj) {
        super(String.CASE_INSENSITIVE_ORDER);
        this.merge(obj);
    }

    public static List<DynObj> sort(List<DynObj> list, String... fields) {
        list.sort(new Comparator<DynObj>() {
            @Override
            public int compare(DynObj o1, DynObj o2) {
                for (String field : fields) {
                    Object fieldValue1 = o1.get(field);
                    Object fieldValue2 = o2.get(field);
                    if (fieldValue1 == null && fieldValue2 == null)
                        continue;

                    if (fieldValue1 != null && fieldValue2 == null)
                        return 1;

                    if (fieldValue1 == null && fieldValue2 != null)
                        return -1;

                    if (fieldValue1 instanceof Comparable) {
                        int v = ((Comparable) fieldValue1).compareTo(fieldValue2);
                        if (v != 0)
                            return v;
                        else
                            continue;
                    }

                    continue;
                }
                return 0;
            }
        });

        return list;
    }

    public DynObj put(String key, Object value) {
        super.put(key, value);
        return this;
    }

    public DynObj mergeObjectField(Object obj) throws IllegalAccessException {
        if (obj instanceof Map) {
            return merge((Map) obj);

        } else {
            Field[] fields = obj.getClass().getDeclaredFields();
            for (Field f : fields) {
                String propertyName = f.getName();
                f.setAccessible(true);
                Object propertyValue = f.get(obj);// ReflectionUtils.getField(f, obj);
                if (propertyValue == null)
                    continue;
                this.put(propertyName, propertyValue);
            }
            return this;
        }
    }

    public DynObj merge(Map obj) {
        if (obj == null)
            return this;

        if (this.equals(obj))
            return this;

        obj.forEach((k, v) -> {
            if (k == null) {
//                this.put("", v);
                return;
            } else {
                this.put(k.toString(), v);
            }
        });

        return this;
    }

    public String getString(String key) {
        Object v = get(key);
        if (v == null) {
            return null;
        }

        if (v instanceof String) {
            return (String) v;
        } else {
            return v.toString();
        }
    }

    public String getString(String key, String defaultValue) {
        Object v = get(key);
        if (v == null) {
            return defaultValue;
        }

        if (v instanceof String) {
            String strV = (String) v;
            if (Strings.isNullOrEmpty(strV)) {
                return defaultValue;
            } else {
                return strV;
            }
        } else {
            return v.toString();
        }
    }

    public int getInt(String key) {
        return getInt(key, 0);
    }

    public int getInt(String key, int defaultValue) {
        Object v = get(key);
        if (v == null) {
            return defaultValue;
        }
        String strValue = v.toString().toLowerCase();
        if (Strings.isNullOrEmpty(strValue)) {
            return defaultValue;
        }
        if (WellKnownRegexValidator.completelyMatches(strValue, WellKnownRegex.INTEGER)) {
            return Integer.parseInt(strValue);
        } else {
            return defaultValue;
        }
    }

    public Boolean boolValue(String fieldName) {
        return boolValue(fieldName, Boolean.FALSE);
    }

    public Boolean boolValue(String fieldName, Boolean defaultValue) {
        if (!this.containsKey(fieldName)) {
            return defaultValue;
        }

        Object v = this.get(fieldName);
        if (v == null) {
            return Boolean.FALSE;
        }

        String strV = v.toString();
        if (strV.equals("0") || strV.equals("false") || strV.equals("False")) {
            return Boolean.FALSE;
        }

        return Boolean.TRUE;
    }

    public String strValue(String fieldName) {
        return strValue(fieldName, "");
    }

    public String strValue(String fieldName, String defaultValue) {
        if (!this.containsKey(fieldName)) {
            return defaultValue;
        }

        Object v = this.get(fieldName);
        if (v == null) {
            return null;
        }

        return v.toString();
    }

    public Integer intValue(String fieldName) {
        return intValue(fieldName, null);
    }

    public Integer intValue(String fieldName, Integer defaultValue) {
        String strValue = strValue(fieldName, null);
        if (Strings.isNullOrEmpty(strValue)) {
            return defaultValue;
        } else {
            return Integer.parseInt(strValue);
        }
    }

    public Double doubleValue(String fieldName) {
        return doubleValue(fieldName, null);
    }

    public Double doubleValue(String fieldName, Double defaultValue) {
        String strValue = strValue(fieldName, null);
        if (Strings.isNullOrEmpty(strValue)) {
            return defaultValue;
        } else {
            return Double.parseDouble(strValue);
        }
    }

    public String description(String... keys) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < keys.length; i++) {
            String k = keys[i];
            String v = getString(k, "");
            if (i > 0) {
                sb.append(", ");
            }
            sb.append(String.format("[%s]:%s", k, v));
        }
        return sb.toString();
    }

    public void mergeTo(Map m) {
        this.forEach((k, v) -> {
            m.put(k, v);
        });
    }

    public String stringByLeves(String keys, String defaultValue) {
        if (Strings.isNullOrEmpty(keys)) {
            return defaultValue;
        }
        int idx = keys.indexOf(".");
        if (idx == 0) {
            return stringByLeves(keys.substring(1), defaultValue);
        } else if (idx == -1) {
            return getString(keys, defaultValue);
        } else {
            String k = keys.substring(0, idx);
            if (!this.containsKey(k)) {
                return defaultValue;
            }
            DynObj obj = new DynObj((Map) this.get(k));
            return obj.stringByLeves(keys.substring(idx + 1), defaultValue);
        }
    }
}
