package pps.core.base.config.excel;

import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.VerticalAlignment;

import java.util.List;


/**
 * excel样式工具
 *
 * @author ZWT
 * @date 2024/03/04
 */
public class ExcelStyleTool {

    /**
     * 写入sheet页
     *
     * @param excelWriter 优秀作家
     * @param sheetNo     图纸编号
     * @param sheetName   图纸名称
     * @param clazz       拍手
     * @param data        数据
     */
    public static void writeSheet(ExcelWriter excelWriter, Integer sheetNo, String sheetName, Class<?> clazz, List data) {
        WriteSheet writeSheet = new WriteSheet();
        writeSheet.setClazz(clazz);
        writeSheet.setSheetNo(sheetNo);
        writeSheet.setSheetName(sheetName);
        excelWriter.write(data, writeSheet);
    }

    /**
     * 设置自定义样式
     *
     * @return {@link HorizontalCellStyleStrategy}
     */
    public static HorizontalCellStyleStrategy getStyleStrategy() {
        //头的策略  样式调整
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        //设置头部样式
        setRowStyle(headWriteCellStyle, true, true);
        //设置细边框
        setBorder(headWriteCellStyle);
        //表头字体样式
        WriteFont headWriteFont = getHeadFont(IndexedColors.SKY_BLUE.getIndex());
        headWriteCellStyle.setWriteFont(headWriteFont);
        //内容的策略
        WriteCellStyle contentStyle = new WriteCellStyle();
        //设置内容样式
        setRowStyle(headWriteCellStyle, false, false);
        //设置边框
        setBorder(contentStyle);
        //内容字体
        contentStyle.setWriteFont(setContentFont());
        //这个策略是 头是头的样式 内容是内容的样式 其他的策略可以自己实现
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentStyle);
    }

    /*-------------------------- 私有方法 --------------------------*/

    /**
     * 获取表头字体
     *
     * @param color
     * @return
     */
    private static WriteFont getHeadFont(Short color) {
        //表头字体样式
        WriteFont headWriteFont = new WriteFont();
        //头字号
        headWriteFont.setFontHeightInPoints((short) 12);
        //字体样式
        headWriteFont.setFontName("微软雅黑");
        //字体颜色
        headWriteFont.setColor(color);
        //字体加粗
        headWriteFont.setBold(true);
        return headWriteFont;
    }

    /**
     * 设置内容字体
     *
     * @return {@link WriteFont}
     */
    private static WriteFont setContentFont() {
        //内容字体
        WriteFont contentWriteFont = new WriteFont();
        //内容字号
        contentWriteFont.setFontHeightInPoints((short) 11);
        //字体样式
        contentWriteFont.setFontName("宋体");
        return contentWriteFont;
    }

    /**
     * 设置行样式
     *
     * @param cellStyle
     * @param wrappedFlag 自动换行标识，true:开启自动换行
     * @param centerFlag  水平居中开关，true:开启水平居中
     */
    private static void setRowStyle(WriteCellStyle cellStyle, boolean wrappedFlag, boolean centerFlag) {
        //行背景:白色
        cellStyle.setFillForegroundColor(IndexedColors.WHITE.getIndex());
        if (wrappedFlag) {
            //设置自动换行
            cellStyle.setWrapped(true);
        }
        if (centerFlag) {
            //设置水平对齐方式
            cellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        }
        //设置垂直对齐方式
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
    }

    /**
     * 设置边框
     *
     * @param cellStyle 单元格样式
     */
    private static void setBorder(WriteCellStyle cellStyle) {
        //设置细边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        cellStyle.setBorderLeft(BorderStyle.THIN);
        cellStyle.setBorderRight(BorderStyle.THIN);
        cellStyle.setBorderTop(BorderStyle.THIN);
        //设置边框颜色 25灰度
        cellStyle.setBottomBorderColor(IndexedColors.BLACK.getIndex());
        cellStyle.setTopBorderColor(IndexedColors.BLACK.getIndex());
        cellStyle.setLeftBorderColor(IndexedColors.BLACK.getIndex());
        cellStyle.setRightBorderColor(IndexedColors.BLACK.getIndex());
    }
}