package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageInput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.IStoragePredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerInput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.SpaceOptimizeLineRelation;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodInput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodOutput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizeWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.error.XError;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 间开优化公用方法
 *
 * @author ZWT
 * @date 2023/09/18 16:02
 */
public class SpaceOptimizeBaseService {

    /**
     * 获取优化详情
     *
     * @param context  上下文
     * @param input    输入
     * @param flag     旗帜
     * @param dateType 日期类型
     * @return {@link GetSpaceOptimizePeriodOutput}
     */
    public GetSpaceOptimizePeriodOutput getSpaceOptimizePeriodView(XContext context, GetSpaceOptimizePeriodInput input, Integer flag, Integer dateType) {
        String periodId = input.getId();
        //查制度详情
        GetSpaceOptimizePeriodOutput output = this.getOptimizePeriodDetail(context, flag, periodId);
        Date optimizeDeadline = Optional.ofNullable(output.getOptimizeDeadline()).orElse(output.getOptimizeDate());
        //设置优化日期列表
        output.setDateList(ServiceUtil.getOptimizeDateList(output.getCreateTime(), optimizeDeadline));
        //查间开制度详情
        SpaceInstitutionDetailEnt detailEnt = this.getInstitutionDetailById(context, output.getInstitutionId());
        //设置电网类型
        output.setGridTypeKey(detailEnt.getGridTypeKey());
        DateTime optimizeDate;
        if (BusinessConstant.FOUR.equals(flag)) {
            //一日优化时间处理
            optimizeDate = DateUtil.beginOfDay(optimizeDeadline);
        } else {
            optimizeDate = DateUtil.beginOfDay(Optional.ofNullable(input.getOptimizeDate())
                    .orElse(output.getCreateTime()));
        }
        //设置市电峰谷策略
        if (StringUtils.equals(String.valueOf(BusinessConstant.ONE), Optional.ofNullable(detailEnt.getGridTypeKey())
                .orElse(String.valueOf(BusinessConstant.ZERO)))) {
            output.setStrategyDetailList(ServiceUtil.getStrategyDetailList(context,
                    GetBasePriceStrategyDetailInput.builder()
                            .lineId(output.getLineId())
                            .strategyMonth(String.valueOf(optimizeDate.monthBaseOne()))
                            .build()));
        } else {
            output.setStrategyDetailList(Collections.emptyList());
        }
        //查功率预测信息
        output.setPowerGenerationList(
                this.getAveragePowerGenerationHourList(context, DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(this.getOptimizeLineRelation(context, CollUtil.newArrayList(detailEnt))
                                .getPlantIdsByLineIdMap()
                                .get(detailEnt.getLineId()))
                        .startTime(optimizeDate.toString())
                        .endTime(DateUtil.offsetDay(optimizeDate, 1).toString())
                        .dateType(dateType)
                        .build())
        );
        //查井口列表
        output.setWellheadList(this.getSpaceOptimizeWellheadList(context, flag, periodId, optimizeDate));
        return output;
    }

    /**
     * 进行优化
     *
     * @param context                上下文
     * @param periodDTOList          时期数据学家
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param dateType               日期类型
     * @param flag                   旗帜
     * @param offset                 抵消
     */
    public void doOptimize(XContext context, List<SpaceOptimizePeriodDTO> periodDTOList, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                           List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                           Integer dateType, Integer flag, Integer offset) {
        //取当前时间
        DateTime date = DateUtil.beginOfDay(DateTime.now());
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = this.getEffectiveSpaceInstitution(context, date.toString(BusinessConstant.DATE_FORMAT_DAY));
        SpaceOptimizeLineRelation relation = this.getOptimizeLineRelation(context, detailEntList);
        //月份要加一
        int monthNum = date.month() + 1;
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //查最新间开历史
            if (this.isOptimize(context, detail.getId(), flag, date)) {
                continue;
            }
            Date institutionEndDate = detail.getInstitutionEndDate();
            DateTime optimizeDeadline = DateUtil.offsetDay(date, offset);
            if (optimizeDeadline.compareTo(institutionEndDate) > 0) {
                optimizeDeadline = DateUtil.endOfDay(institutionEndDate);
            }
            //创建记录
            String periodId = this.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(), BaseUtils.getExecutionCycleForCalibration(date, DateUtil.offsetDay(optimizeDeadline, -1)), optimizeDeadline);
            for (int i = 0; i < offset; i++) {
                DateTime startDate = DateUtil.offsetDay(date, i);
                DateTime optimizeDate = DateUtil.offsetDay(date, i + 1);
                DynamicQueryPlantPredictedPowerInput powerInput = DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                        .startTime(startDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .endTime(optimizeDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                        .dateType(dateType)
                        .build();
                switch (detail.getGridTypeKey()) {
                    //并网型优化
                    case "1":
                        this.gridConnectedOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                                relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()), detail, monthNum, periodId,
                                powerInput, offset > 1 ? startDate : optimizeDate
                        );
                        break;
                    //离网型优化
                    case "0":
                        this.offGridOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                                relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()), relation.getStorageAvgMap(), detail, periodId,
                                DynamicQueryStoragePredictedPowerInput.builder()
                                        .storageIds(relation.getStorageIdsByLineIdMap().get(detail.getLineId()))
                                        .startTime(DateUtil.offsetDay(startDate, -offset).toString(BusinessConstant.DATE_FORMAT_DAY))
                                        .endTime(DateUtil.offsetDay(optimizeDate, -offset).toString(BusinessConstant.DATE_FORMAT_DAY))
                                        .build(), powerInput, offset > 1 ? startDate : optimizeDate
                        );
                        break;
                    default:
                        //电网类型不存在
                }
            }
        }
    }

    /**
     * 获得生效中的基础间开
     *
     * @param context 上下文
     * @param sameDay 同一天
     * @return {@link List}<{@link SpaceInstitutionDetailEnt}>
     */
    public List<SpaceInstitutionDetailEnt> getEffectiveSpaceInstitution(XContext context, String sameDay) {
        //取生效中的基础间开
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        List<SpaceInstitutionDetailEnt> detailEntList = detailMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                .le(SpaceInstitutionDetailEnt::getInstitutionStartDate, sameDay)
                .ge(SpaceInstitutionDetailEnt::getInstitutionEndDate, sameDay)
        );
        if (CollUtil.isEmpty(detailEntList)) {
            //未发现可优化基础间开制度
            throw new XServiceException(BusinessError.DidNotFindSpace);
        }
        return detailEntList;
    }

    /**
     * 获取所有生效中制度关联的井口及光伏电站
     *
     * @param context       上下文
     * @param detailEntList 详细ent列表
     * @return {@link SpaceOptimizeLineRelation}
     */
    public SpaceOptimizeLineRelation getOptimizeLineRelation(XContext context, List<SpaceInstitutionDetailEnt> detailEntList) {
        Set<String> lineIds = new HashSet<>(32);
        Set<String> institutionIds = new HashSet<>(32);
        for (SpaceInstitutionDetailEnt spaceInstitutionDetailEnt : detailEntList) {
            lineIds.add(spaceInstitutionDetailEnt.getLineId());
            institutionIds.add(spaceInstitutionDetailEnt.getId());
        }
        //取生效中间开的所有井口
        SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadMapper.selectListByInstitutionIds(institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            //未发现可优化井口
            throw new XServiceException(BusinessError.DidNotFindWellhead);
        }
        this.setServiceRatingForSpaceWellheadList(context, spaceWellheadList);
        return SpaceOptimizeLineRelation.builder()
                .wellheadsByInstitutionIdIdMap(spaceWellheadList.stream()
                        .collect(Collectors.groupingBy(SpaceInstitutionWellheadView::getInstitutionId)))
                .plantIdsByLineIdMap(
                        //取线路关联的所有光伏电站ID
                        this.getPowerLinePlantListByLineIds(context, lineIds).stream()
                                .collect(
                                        Collectors.groupingBy(DynamicQueryBasePowerLinePlantOutput::getLineId,
                                                Collectors.mapping(DynamicQueryBasePowerLinePlantOutput::getPlantId, Collectors.toList()))
                                )
                )
                //取线路关联所有储能电站计算平均功率
                .storageAvgMap(this.getPowerLineStorageAvgByLineIds(context, lineIds))
                .storageIdsByLineIdMap(this.getPowerLineStorageListByLineIds(context, lineIds))
                .build();
    }

    /**
     * 并网优化
     *
     * @param context                上下文
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param spaceWellheadList      空间井口清单
     * @param detail                 细节
     * @param monthNum               月份
     * @param periodId               期间id
     * @param plantPowerInput        电厂功率输入
     * @param optimizeDate           优化日期
     */
    public void gridConnectedOptimization(XContext context, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                                          List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                          List<SpaceInstitutionWellheadView> spaceWellheadList,
                                          SpaceInstitutionDetailEnt detail, int monthNum, String periodId,
                                          DynamicQueryPlantPredictedPowerInput plantPowerInput, DateTime optimizeDate) {
        //获取当前制度对应的光伏预测数据列表
        Map<String, List<SpaceInstitutionDurationEnt>> durationMap = this.getDurationMap(context, detail.getId());
        List<DynamicQueryPlantPredictedPowerOutput> avgPowerList = this.getAveragePowerGenerationListByPlantIds(context, plantPowerInput);
        if (CollUtil.isEmpty(avgPowerList)) {
            //没有预测数据都不优化
            this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                    spaceWellheadList, durationMap, periodId, optimizeDate);
            return;
        }
        //取光伏出力峰值
        BigDecimal powerMax = this.getPowerMax(avgPowerList);
        //取当前制度下井口的总功率
        BigDecimal wellheadTotalPower = this.getWellheadTotalPower(spaceWellheadList, detail.getId());
        //根据类型过滤井口:大间开,连抽井不优化
        Map<Boolean, List<SpaceInstitutionWellheadView>> wellheadViewMap = this.getWellheadViewList(spaceWellheadList, detail.getId());
        //保存不需要优化的井口
        this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                wellheadViewMap.get(false), durationMap, periodId, optimizeDate);
        List<SpaceInstitutionWellheadView> wellheadViewList = wellheadViewMap.get(true);
        if (CollUtil.isEmpty(wellheadViewList)) {
            return;
        }
        if (CollUtil.isEmpty(durationMap)) {
            //没有设置时间段,无法优化
            return;
        }
        //光伏出力峰值大于等于井口总功率:绿电消纳优先策略
        if (powerMax.compareTo(wellheadTotalPower) >= BusinessConstant.ZERO) {
            //---------------------------------绿电消纳策略---------------------------------
            this.greenElectricityConsumptionStrategy(wellheadDTOList, unOptimizeDurationList, durationDTOList,
                    wellheadViewList, avgPowerList, durationMap, detail, periodId, optimizeDate);
        } else {
            //遍历井口,按发电功率大于等于光伏出力峰值条件分组
            Map<Boolean, List<SpaceInstitutionWellheadView>> collect = this.getWellheadViewMapByPower(wellheadViewList, powerMax);
            List<SpaceInstitutionWellheadView> lowWellheadList = collect.get(false);
            int size = spaceWellheadList.size();
            int lowWellheadListSize = lowWellheadList.size();
            //光伏出力峰值<任何一口井的运行功率:消峰平谷策略
            if (size == lowWellheadListSize) {
                //---------------------------------消峰平谷策略---------------------------------
                this.peakEliminationAndValleyLevelingStrategy(context, wellheadDTOList, unOptimizeDurationList, durationDTOList, wellheadViewList,
                        durationMap, detail, monthNum, periodId, optimizeDate);
            }
            //光伏出力峰值>=线路哪部分井口运行功率:满足的部分井口采用绿电消纳优先,不满足的井采用消峰平谷
            else {
                List<SpaceInstitutionWellheadView> highWellheadList = collect.get(true);
                if (CollUtil.isNotEmpty(highWellheadList)) {
                    this.greenElectricityConsumptionStrategy(wellheadDTOList, unOptimizeDurationList, durationDTOList,
                            highWellheadList, avgPowerList, durationMap, detail, periodId, optimizeDate);
                }
                this.peakEliminationAndValleyLevelingStrategy(context, wellheadDTOList, unOptimizeDurationList, durationDTOList,
                        lowWellheadList, durationMap, detail, monthNum, periodId, optimizeDate);
            }
        }
    }

    /**
     * 离网优化
     *
     * @param context                上下文
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param spaceWellheadList      空间井口清单
     * @param storageAvgMap          存储平均值映射
     * @param detail                 细节
     * @param periodId               期间id
     * @param input                  输入
     * @param plantPowerInput        电厂功率输入
     * @param optimizeDate           优化日期
     */
    public void offGridOptimization(XContext context, List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceOptimizeDurationDTO> durationDTOList,
                                    List<SpaceOptimizeDurationDTO> unOptimizeDurationList, List<SpaceInstitutionWellheadView> spaceWellheadList,
                                    Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> storageAvgMap, SpaceInstitutionDetailEnt detail, String periodId,
                                    DynamicQueryStoragePredictedPowerInput input, DynamicQueryPlantPredictedPowerInput plantPowerInput, DateTime optimizeDate) {
        //获取井口间开时间段
        Map<String, List<SpaceInstitutionDurationEnt>> durationMap = this.getDurationMap(context, detail.getId());
        //获取当前制度对应的光伏预测数据列表
        List<DynamicQueryPlantPredictedPowerOutput> avgPowerList = this.getAveragePowerGenerationListByPlantIds(context, plantPowerInput);
        if (CollUtil.isEmpty(avgPowerList)) {
            //没有预测数据都不优化
            this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                    spaceWellheadList, durationMap, periodId, optimizeDate);
            return;
        }
        //根据类型过滤井口:大间开,连抽井不优化
        Map<Boolean, List<SpaceInstitutionWellheadView>> wellheadViewMap = this.getWellheadViewList(spaceWellheadList, detail.getId());
        //保存不需要优化的井口
        this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                wellheadViewMap.get(false), durationMap, periodId, optimizeDate);
        List<SpaceInstitutionWellheadView> wellheadViewList = wellheadViewMap.get(true);
        if (CollUtil.isEmpty(wellheadViewList)) {
            return;
        }
        if (CollUtil.isEmpty(durationMap)) {
            //没有设置时间段,无法优化
            return;
        }
        //时间差
        int between = 0;
        //启动间隔累积
        int startInterval = 0;
        //判断是否第一口井
        boolean isFirstWellhead;
        //第一口井启动时间
        DateTime firstStartTime = null;
        //第一口井开井时间
        int firstIndex = 0;
        //井口累积运行总功率
        BigDecimal totalOperatingPower = BigDecimal.ZERO;
        //离网时间段
        List<SpaceOptimizeDurationDTO> offGridPeriodList = new ArrayList<>(32);
        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
            String wellheadId = wellhead.getWellheadId();
            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
            //取当前井口最大发电量
            BigDecimal serviceRating = wellhead.getServiceRating();
            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellheadId);
            if (CollUtil.isEmpty(durationConfigList)) {
                //没有设置时间段,无法优化
                continue;
            }
            //保存原始记录
            for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
            }
            totalOperatingPower = totalOperatingPower.add(serviceRating);
            if (w == 0) {
                //第一个井口
                isFirstWellhead = true;
            } else {
                //累加
                startInterval = startInterval + detail.getStartInterval();
                isFirstWellhead = false;
            }
            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                DateTime startTime = DateUtil.date(duration.getOpenTime());
                Date closeTime = duration.getCloseTime();
                DateTime endTime;
                if (ObjectUtil.isNull(closeTime)) {
                    endTime = DateUtil.beginOfDay(DateUtil.tomorrow());
                } else {
                    endTime = DateUtil.date(closeTime);
                }
                //第一次启动
                if (d == 0) {
                    //第一口井的启动时间
                    if (isFirstWellhead) {
                        //计算开井时间
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            //当日时间段平均光伏出力>=第一口井运行负荷时,该时间为第一口井运行时间
                            if (predictedPower.getPower().compareTo(serviceRating) >= BusinessConstant.ZERO) {
                                firstIndex = a;
                                break;
                            }
                        }
                        DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(firstIndex);
                        DateTime startTimeOptimize;
                        DateTime endTimeOptimize;
                        //判断发电量是否为正数
                        if (start.getPower().compareTo(BigDecimal.ZERO) > 0) {
                            startTimeOptimize = DateUtil.date(start.getCreateTime());
                            firstStartTime = startTimeOptimize;
                            //计算第一次关井时间,按照间开时间段顺延
                            int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                            endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, startDuration);
                        } else {
                            startTimeOptimize = startTime;
                            firstStartTime = startTime;
                            endTimeOptimize = endTime;
                        }
                        this.createDurationOffGridPeriod(offGridPeriodList, wellheadId, startTimeOptimize, endTimeOptimize, BusinessConstant.ZERO);
                        //计算时间偏移,取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(startTimeOptimize, startTime);
                    }
                    //其它井口的第一次启动时间
                    else {
                        DateTime startTimeOffset = firstStartTime.offsetNew(DateField.MINUTE, startInterval);
                        int startIndex = -1;
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            if (DateUtil.date(predictedPower.getCreateTime()).compareTo(startTimeOffset) >= 0) {
                                //判断第一口井启动时间+启动间隔时日平均光伏出力-前两口井的运行功率是否为正数
                                if (predictedPower.getPower().compareTo(totalOperatingPower) >= 0) {
                                    //确定第二口井第一次开井时间为第一口井启动时间+启动间隔
                                    startIndex = a;
                                    break;
                                }
                            }
                        }
                        DateTime startTimeOptimize;
                        DateTime endTimeOptimize;
                        //计算未优化启动间隔
                        int openDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                        if (startIndex > -1) {
                            DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                            startTimeOptimize = DateUtil.date(start.getCreateTime());
                            endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, openDuration);
                        } else {
                            //无法优化
                            endTimeOptimize = startTimeOffset.offsetNew(DateField.MINUTE, openDuration);
                            startTimeOptimize = startTimeOffset;
                        }
                        this.createDurationOffGridPeriod(offGridPeriodList, wellheadId, startTimeOptimize, endTimeOptimize, BusinessConstant.ZERO);
                        //取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                        //修改启动时间
                        firstStartTime = startTimeOptimize;
                    }
                } else {
                    //计算偏移
                    this.createDurationOffGridPeriod(offGridPeriodList, wellheadId,
                            DateUtil.offsetMinute(startTime, between),
                            DateUtil.offsetMinute(endTime, between), BusinessConstant.ONE
                    );
                }
            }
        }
        //离网优化
        if (CollUtil.isNotEmpty(offGridPeriodList)) {
//            BigDecimal avgRatedDischargeEfficiency = BigDecimal.ZERO;
//            BigDecimal avgRatedDischargeDepth = BigDecimal.ZERO;
//            BigDecimal avgRatedDischargePower = BigDecimal.ZERO;
//            Iterator<DynamicQueryStoragePredictedPowerOutput> iterator = null;
//            if (storageAvgMap.containsKey(detail.getLineId())) {
//                //查获取前一个月该时间点储能剩余电量，求平均数
//                List<DynamicQueryStoragePredictedPowerOutput> averageEnergyStorageList = this.getAverageEnergyStorageListByParam(context, input);
//                if (CollUtil.isEmpty(averageEnergyStorageList)) {
//                    //没有储能发电量,无法计算
//                    return;
//                }
//                //取储能计算参数
//                List<DynamicQueryBasePowerLineStorageViewOutput> avgStorageViewList = storageAvgMap.get(detail.getLineId());
//                DynamicQueryBasePowerLineStorageViewOutput avgStorageView = avgStorageViewList.get(0);
//                //平均额定放电效率(%)
//                avgRatedDischargeEfficiency = avgStorageView.getRatedDischargeEfficiency();
//                //平均额定放电深度(%)
//                avgRatedDischargeDepth = avgStorageView.getRatedDischargeDepth();
//                //平均额定放电功率(KW)
//                avgRatedDischargePower = avgStorageView.getRatedDischargePower();
//                //逐15分钟计算光伏出力-井场运行功率之和，获得光伏出力不足时间段,并计算储能可用时长
//                iterator = averageEnergyStorageList.iterator();
//            }
//            //截取从第一次开井时间往后的时间段
//            List<DynamicQueryPlantPredictedPowerOutput> subAvgPowerList = CollUtil.sub(avgPowerList, firstIndex, avgPowerList.size());
//            if (CollUtil.isEmpty(subAvgPowerList)) {
//                //没有时间段,无法计算
//                return;
//            }
//            for (DynamicQueryPlantPredictedPowerOutput avg : subAvgPowerList) {
//                if (totalOperatingPower.compareTo(avg.getPower()) >= 0) {
//                    //电量不足,计算储能可用时长
//                    avg.setBatteryLowFlag(false);
//                    String hourTime = avg.getHourTime();
//                    String minTime = avg.getMinTime();
//                    if (Objects.isNull(iterator)) {
//                        avg.setPowerDuration(0);
//                    } else {
//                        while (iterator.hasNext()) {
//                            DynamicQueryStoragePredictedPowerOutput storagePredicted = iterator.next();
//                            if (StringUtils.equals(hourTime, storagePredicted.getHourTime()) && StringUtils.equals(minTime, storagePredicted.getMinTime())) {
//                                //计算储能可供电时长:（储能剩余容量*放电深度*放电效率）/额定放电功率
//                                avg.setPowerDuration(
//                                        storagePredicted.getPower()
//                                                .multiply(avgRatedDischargeDepth)
//                                                .multiply(avgRatedDischargeEfficiency)
//                                                .divide(avgRatedDischargePower, 6, BigDecimal.ROUND_HALF_UP)
//                                                .setScale(BusinessConstant.ZERO, RoundingMode.DOWN)
//                                                .intValueExact()
//                                );
//                                break;
//                            }
//                            //删除计算过的时间段
//                            iterator.remove();
//                        }
//                    }
//                } else {
//                    avg.setBatteryLowFlag(true);
//                }
//            }
            //拆分时间段
            for (SpaceOptimizeDurationDTO durationDTO : offGridPeriodList) {
                //-----------------------------------大庆暂时不用储能
                DateTime openTime = DateUtil.date(durationDTO.getOpenTime());
                DateTime closeTime = DateUtil.date(durationDTO.getCloseTime());
                if (openTime.compareTo(BusinessConstant.DATE_FLAG) < 0 && closeTime.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            openTime.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                            durationDTO.getFirstStart(), optimizeDate
                    );
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            BusinessConstant.START_OF_DAY_TIME, closeTime.toString(BusinessConstant.MINUTES_FORMAT),
                            durationDTO.getFirstStart(), optimizeDate
                    );
                } else if (closeTime.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            openTime.toString(BusinessConstant.MINUTES_FORMAT), BusinessConstant.END_OF_DAY_TIME,
                            durationDTO.getFirstStart(), optimizeDate
                    );
                } else {
                    this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
                            openTime.toString(BusinessConstant.MINUTES_FORMAT), closeTime.toString(BusinessConstant.MINUTES_FORMAT),
                            durationDTO.getFirstStart(), optimizeDate
                    );
                }
                //-----------------------------------原始逻辑
//                Date openTime = durationDTO.getOpenTime();
//                Date closeTime = durationDTO.getCloseTime();
//                if (closeTime.compareTo(BusinessConstant.DATE_FLAG) > 0) {
//                    closeTime = DateUtil.endOfDay(DateUtil.date());
//                }
                //匹配时间
//                for (DynamicQueryPlantPredictedPowerOutput avg : subAvgPowerList) {
//                    DateTime createTime = DateUtil.date(avg.getCreateTime());
//                    //计算时间范围:大于等于开始时间,小于等于结束时间
//                    if (createTime.compareTo(openTime) >= 0 && createTime.compareTo(closeTime) < 0) {
//                        String endString = createTime.offsetNew(DateField.MINUTE, 15).toString(BusinessConstant.MINUTES_FORMAT);
//                        String startString = createTime.toString(BusinessConstant.MINUTES_FORMAT);
//                        if (StringUtils.equals(endString, BusinessConstant.START_OF_DAY_TIME)) {
//                            endString = BusinessConstant.END_OF_DAY_TIME;
//                        }
//                        //电量满足跳过
//                        if (avg.getBatteryLowFlag()) {
//                            //创建光伏
//                            this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.PHOTOVOLTAIC,
//                                    startString, endString
//                            );
//                        } else {
//                            int powerDuration = avg.getPowerDuration();
//                            powerDuration = Math.min(powerDuration, 15);
//                            String offset = createTime.offsetNew(DateField.MINUTE, powerDuration).toString(BusinessConstant.MINUTES_FORMAT);
//                            if (StringUtils.equals(offset, BusinessConstant.START_OF_DAY_TIME)) {
//                                offset = BusinessConstant.END_OF_DAY_TIME;
//                            }
//                            if (0 != powerDuration) {
//                                //储能发电时长
//                                this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.STORED_ENERGY,
//                                        startString, offset
//                                );
//                            }
//                            if (15 != powerDuration) {
//                                //柴发发电时长
//                                this.createOptimizeDuration(durationDTOList, periodId, durationDTO.getRecordId(), durationDTO.getWellheadId(), BusinessConstant.DIESEL_POWER,
//                                        offset, endString
//                                );
//                            }
//                        }
//                    }
//                }
            }
            //时间段优化
//            if (CollUtil.isNotEmpty(durationDTOList)) {
//                List<SpaceOptimizeDurationDTO> finallyList = new ArrayList<>(durationDTOList.size());
//                List<SpaceOptimizeDurationDTO> offList = new ArrayList<>(durationDTOList.size());
//                for (SpaceOptimizeDurationDTO durationDTO : durationDTOList) {
//                    if (StringUtils.equals(periodId, durationDTO.getPeriodId())) {
//                        offList.add(durationDTO);
//                    } else {
//                        finallyList.add(durationDTO);
//                    }
//                }
//                //优化
//                if (CollUtil.isNotEmpty(offList)) {
//                    Map<String, List<SpaceOptimizeDurationDTO>> collect = offList.stream()
//                            .collect(Collectors.groupingBy(SpaceOptimizeDurationDTO::getWellheadId));
//                    for (String wellheadId : collect.keySet()) {
//                        List<SpaceOptimizeDurationDTO> list = collect.get(wellheadId).stream()
//                                .sorted(Comparator.comparing(SpaceOptimizeDurationDTO::getOpenWellTime))
//                                .collect(Collectors.toList());
//                        int size = list.size();
//                        if (size == 1) {
//                            finallyList.add(list.get(0));
//                            continue;
//                        }
//                        List<SpaceOptimizeDurationDTO> temporarilyList = CollUtil.newArrayList(list.get(0));
//                        for (int i = 1; i < size; i++) {
//                            SpaceOptimizeDurationDTO end = temporarilyList.get(temporarilyList.size() - 1);
//                            SpaceOptimizeDurationDTO durationDTO = list.get(i);
//                            boolean equals = StringUtils.equals(end.getGenerationTypeKey(), durationDTO.getGenerationTypeKey());
//                            //判断类型和时间段
//                            if (!equals) {
//                                SpaceOptimizeDurationDTO begin = temporarilyList.get(0);
//                                begin.setCloseWellTime(end.getCloseWellTime());
//                                finallyList.add(begin);
//                                temporarilyList = new ArrayList<>(12);
//                            } else if (!StringUtils.equals(end.getCloseWellTime(), durationDTO.getOpenWellTime()) && !StringUtils.equals(end.getId(), durationDTO.getId())) {
//                                SpaceOptimizeDurationDTO begin = temporarilyList.get(0);
//                                begin.setCloseWellTime(end.getCloseWellTime());
//                                finallyList.add(begin);
//                                temporarilyList = new ArrayList<>(12);
//                            }
//                            temporarilyList.add(durationDTO);
//                            if (i == size - 1) {
//                                SpaceOptimizeDurationDTO begin = temporarilyList.get(0);
//                                begin.setCloseWellTime(durationDTO.getCloseWellTime());
//                                finallyList.add(begin);
//                            }
//                        }
//                    }
//                    CollUtil.clear(durationDTOList);
//                    durationDTOList.addAll(finallyList);
//                }
//            }
        }
    }

    /**
     * 获取每小时平均发电量列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    public List<DynamicQueryPlantPredictedPowerOutput> getAveragePowerGenerationHourList(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        IPlantPredictedPowerCloudService cloudService = context.getBean(IPlantPredictedPowerCloudService.class);
        XListResult<DynamicQueryPlantPredictedPowerOutput> result = cloudService.queryAveragePowerGenerationHourListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 创建间开优化信息
     *
     * @param periodDTOList          时期数据主义者
     * @param detailId               详细信息id
     * @param lineId                 线路id
     * @param executionCycleForMonth 月执行周期
     * @param optimizeDeadline       优化截止日期
     * @return {@link String}
     */
    public String createOptimizePeriod(List<SpaceOptimizePeriodDTO> periodDTOList, String detailId,
                                       String lineId, String executionCycleForMonth, Date optimizeDeadline) {
        SpaceOptimizePeriodDTO periodDTO = new SpaceOptimizePeriodDTO();
        BaseUtils.setBaseModelDefaultForJob(periodDTO);
        periodDTO.setInstitutionId(detailId);
        periodDTO.setLineId(lineId);
        periodDTO.setExecutionCycle(executionCycleForMonth);
        periodDTO.setOptimizeState(BusinessConstant.ZERO);
        periodDTO.setOptimizeDeadline(optimizeDeadline);
        periodDTOList.add(periodDTO);
        return periodDTO.getId();
    }

    /**
     * 创建间开优化井口信息
     *
     * @param wellheadDTOList 井口dtolist
     * @param periodId        期间id
     * @param wellheadId      井口id
     * @param wellNumber      井号
     * @param startSeq        开始seq
     * @param optimizeDate    优化日期
     * @param frostproofWell  防冻井
     * @return {@link String}
     */
    public String createOptimizeWellhead(List<SpaceOptimizeWellheadDTO> wellheadDTOList, String periodId, String wellheadId,
                                         String wellNumber, Integer startSeq, DateTime optimizeDate, Integer frostproofWell) {
        SpaceOptimizeWellheadDTO wellheadDTO = new SpaceOptimizeWellheadDTO();
        BaseUtils.setBaseModelDefaultForJob(wellheadDTO);
        wellheadDTO.setPeriodId(periodId);
        wellheadDTO.setWellheadId(wellheadId);
        wellheadDTO.setWellNumber(wellNumber);
        wellheadDTO.setStartSeq(startSeq);
        wellheadDTO.setOptimizeDate(optimizeDate);
        wellheadDTO.setFrostproofWell(frostproofWell);
        wellheadDTOList.add(wellheadDTO);
        return wellheadDTO.getId();
    }

    /**
     * 创建间开原始间开区间
     *
     * @param unOptimizeDurationList 取消优化工期列表
     * @param durationEnt            持续时间ent
     * @param periodId               期间id
     * @param recordId               记录id
     * @param wellheadId             井口id
     * @param optimizeDate           优化时间
     */
    public void createUnOptimizeDuration(List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                         SpaceInstitutionDurationEnt durationEnt,
                                         String periodId, String recordId, String wellheadId, DateTime optimizeDate) {
        this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE,
                durationEnt.getOpenWellTime(), durationEnt.getCloseWellTime(), null,
                BusinessConstant.ONE, optimizeDate);
    }

    /**
     * 创建间开优化井口区间配置信息
     *
     * @param durationDTOList   持续时间dtolist
     * @param periodId          期间id
     * @param recordId          记录id
     * @param wellheadId        井口id
     * @param generationTypeKey 生成类型密钥
     * @param openWellTime      开井时间
     * @param endTimeString     结束时间字符串
     * @param firstStart        是否第一次启动
     * @param optimizeDate      优化时间
     */
    public void createOptimizeDuration(List<SpaceOptimizeDurationDTO> durationDTOList, String periodId,
                                       String recordId, String wellheadId, String generationTypeKey,
                                       String openWellTime, String endTimeString, Integer firstStart, DateTime optimizeDate) {
        this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO,
                openWellTime, endTimeString, generationTypeKey, firstStart, optimizeDate);
    }

    /**
     * 创建持续时间
     *
     * @param durationDTOList   持续时间dtolist
     * @param recordId          记录id
     * @param periodId          期间id
     * @param wellheadId        井口id
     * @param isOptimize        正在优化
     * @param openWellTime      开井时间
     * @param closeWellTime     关井时间
     * @param generationTypeKey 生成类型密钥
     * @param firstStart        是否第一次启动
     * @param optimizeDate      优化时间
     */
    public void createDuration(List<SpaceOptimizeDurationDTO> durationDTOList, String recordId,
                               String periodId, String wellheadId, Integer isOptimize, String openWellTime,
                               String closeWellTime, String generationTypeKey, Integer firstStart, DateTime optimizeDate) {
        SpaceOptimizeDurationDTO durationDTO = new SpaceOptimizeDurationDTO();
        BaseUtils.setBaseModelDefaultForJob(durationDTO);
        durationDTO.setRecordId(recordId);
        durationDTO.setPeriodId(periodId);
        durationDTO.setWellheadId(wellheadId);
        durationDTO.setIsOptimize(isOptimize);
        durationDTO.setOpenWellTime(openWellTime);
        durationDTO.setCloseWellTime(closeWellTime);
        durationDTO.setGenerationTypeKey(generationTypeKey);
        durationDTO.setFirstStart(firstStart);
        durationDTO.setOptimizeDate(optimizeDate);
        durationDTOList.add(durationDTO);
    }

    /**
     * 条件获取获取光伏预测各时段平均值列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    public List<DynamicQueryPlantPredictedPowerOutput> getAveragePowerGenerationListByPlantIds(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        IPlantPredictedPowerCloudService cloudService = context.getBean(IPlantPredictedPowerCloudService.class);
        XListResult<DynamicQueryPlantPredictedPowerOutput> result = cloudService.queryAveragePowerGenerationListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获得半小时平均功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    public List<DynamicQueryPlantPredictedPowerOutput> getAveragePowerHalfHour(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        IPlantPredictedPowerCloudService cloudService = context.getBean(IPlantPredictedPowerCloudService.class);
        XListResult<DynamicQueryPlantPredictedPowerOutput> result = cloudService.queryAveragePowerHalfHour(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 按id获取间开详细信息
     *
     * @param context       上下文
     * @param institutionId 机构id
     * @return {@link SpaceInstitutionDetailEnt}
     */
    private SpaceInstitutionDetailEnt getInstitutionDetailById(XContext context, String institutionId) {
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        return detailMapper.selectById(institutionId);
    }

    /**
     * 获取优化期间详细信息
     *
     * @param context  上下文
     * @param flag     旗帜
     * @param periodId 期间id
     * @return {@link GetSpaceOptimizePeriodOutput}
     */
    private GetSpaceOptimizePeriodOutput getOptimizePeriodDetail(XContext context, Integer flag, String periodId) {
        GetSpaceOptimizePeriodOutput output = null;
        switch (flag) {
            case 1:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeUltraPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            case 2:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeLongPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            case 3:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeMidPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            case 4:
                output = XCopyUtils.copyNewObject(context.getBean(SpaceOptimizeShortPeriodMapper.class).selectById(periodId),
                        GetSpaceOptimizePeriodOutput.class);
                break;
            default:
        }
        if (ObjectUtil.isNull(output)) {
            throw new XServiceException(context, XError.NotFound);
        }
        return output;
    }

    /**
     * 获取空间优化井口列表
     *
     * @param context      上下文
     * @param flag         旗帜
     * @param periodId     期间id
     * @param optimizeDate 优化日期
     * @return {@link List}<{@link GetSpaceOptimizeWellheadOutput}>
     */
    private List<GetSpaceOptimizeWellheadOutput> getSpaceOptimizeWellheadList(XContext context, Integer flag, String periodId, DateTime optimizeDate) {
        //井口列表
        List<GetSpaceOptimizeWellheadOutput> wellheadOutputs = null;
        //配置列表
        List<SpaceOptimizeDateDuration> durationOutputs = null;
        switch (flag) {
            case 1:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeUltraWellheadMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeUltraWellheadEnt>()
                        .eq(SpaceOptimizeUltraWellheadEnt::getUltraPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeUltraWellheadEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeUltraWellheadEnt::getStartSeq)
                ), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeUltraDurationMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeUltraDurationEnt>()
                        .eq(SpaceOptimizeUltraDurationEnt::getUltraPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeUltraDurationEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeUltraDurationEnt::getIsOptimize, SpaceOptimizeUltraDurationEnt::getOpenWellTime)
                ), SpaceOptimizeDateDuration.class);
                break;
            case 2:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeLongWellheadMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeLongWellheadEnt>()
                        .eq(SpaceOptimizeLongWellheadEnt::getLongPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeLongWellheadEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeLongWellheadEnt::getStartSeq)
                ), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeLongDurationMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeLongDurationEnt>()
                        .eq(SpaceOptimizeLongDurationEnt::getLongPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeLongDurationEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeLongDurationEnt::getIsOptimize, SpaceOptimizeLongDurationEnt::getOpenWellTime)
                ), SpaceOptimizeDateDuration.class);
                break;
            case 3:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeMidWellheadMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeMidWellheadEnt>()
                        .eq(SpaceOptimizeMidWellheadEnt::getMidPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeMidWellheadEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeMidWellheadEnt::getStartSeq)
                ), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeMidDurationMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeMidDurationEnt>()
                        .eq(SpaceOptimizeMidDurationEnt::getMidPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeMidDurationEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeMidDurationEnt::getIsOptimize, SpaceOptimizeMidDurationEnt::getOpenWellTime)
                ), SpaceOptimizeDateDuration.class);
                break;
            case 4:
                wellheadOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeShortWellheadMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                        .eq(SpaceOptimizeShortWellheadEnt::getShortPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeShortWellheadEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeShortWellheadEnt::getStartSeq)
                ), GetSpaceOptimizeWellheadOutput.class);
                durationOutputs = XCopyUtils.copyNewList(context.getBean(SpaceOptimizeShortDurationMapper.class).selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                        .eq(SpaceOptimizeShortDurationEnt::getShortPeriodId, periodId)
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, optimizeDate)
                        .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
                ), SpaceOptimizeDateDuration.class);
                break;
            default:
        }
        if (CollUtil.isEmpty(wellheadOutputs)) {
            wellheadOutputs = Collections.emptyList();
        } else {
            if (CollUtil.isNotEmpty(durationOutputs)) {
                Map<String, List<SpaceOptimizeDateDuration>> collect = new HashMap<>(16);
                for (SpaceOptimizeDateDuration durationOutput : durationOutputs) {
                    if (StringUtils.isEmpty(durationOutput.getGenerationTypeKey())) {
                        durationOutput.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                    }
                    List<SpaceOptimizeDateDuration> tmpList = collect.get(durationOutput.getWellheadId());
                    if (CollUtil.isEmpty(tmpList)) {
                        tmpList = new ArrayList<>(16);
                        tmpList.add(durationOutput);
                        collect.put(durationOutput.getWellheadId(), tmpList);
                    } else {
                        tmpList.add(durationOutput);
                    }
                }
                wellheadOutputs.forEach(w -> {
                    if (collect.containsKey(w.getWellheadId())) {
                        Map<Integer, List<SpaceOptimizeDateDuration>> listMap = collect.get(w.getWellheadId()).stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getIsOptimize));
                        w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(0)));
                        w.setInitializeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                    } else {
                        w.setOptimizeList(new ArrayList<>(0));
                        w.setInitializeList(new ArrayList<>(0));
                    }
                });
            }
        }
        return wellheadOutputs;
    }

    /**
     * 绿电消纳策略
     *
     * @param wellheadDTOList        井口dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param durationDTOList        持续时间dtolist
     * @param wellheadViewList       井口视图列表
     * @param avgPowerList           平均功率列表
     * @param durationMap            持续时间图
     * @param detail                 细节
     * @param periodId               期间id
     * @param optimizeDate           优化日期
     */
    private void greenElectricityConsumptionStrategy(List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                                     List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceInstitutionWellheadView> wellheadViewList,
                                                     List<DynamicQueryPlantPredictedPowerOutput> avgPowerList, Map<String, List<SpaceInstitutionDurationEnt>> durationMap,
                                                     SpaceInstitutionDetailEnt detail, String periodId, DateTime optimizeDate) {
        //时间差
        int between = 0;
        //启动间隔累积
        int startInterval = 0;
        //判断是否第一口井
        boolean isFirstWellhead;
        //第一口井启动时间
        DateTime firstStartTime = null;
        //井口累积运行总功率
        BigDecimal totalOperatingPower = BigDecimal.ZERO;
        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
            String wellheadId = wellhead.getWellheadId();
            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
            //取当前井口最大发电量
            BigDecimal serviceRating = wellhead.getServiceRating();
            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellheadId);
            if (CollUtil.isEmpty(durationConfigList)) {
                //没有设置时间段,无法优化
                continue;
            }
            //保存原始记录
            for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
            }
            //累加
            totalOperatingPower = totalOperatingPower.add(serviceRating);
            if (w == 0) {
                //第一个井口
                isFirstWellhead = true;
            } else {
                startInterval = startInterval + detail.getStartInterval();
                isFirstWellhead = false;
            }
            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                DateTime startTime = DateUtil.date(duration.getOpenTime());
                Date closeTime = duration.getCloseTime();
                DateTime endTime;
                if (ObjectUtil.isNull(closeTime)) {
                    endTime = DateUtil.beginOfDay(DateUtil.tomorrow());
                } else {
                    endTime = DateUtil.date(closeTime);
                }
                //第一次启动
                if (d == 0) {
                    //第一口井的启动时间
                    if (isFirstWellhead) {
                        //计算开井时间
                        int startIndex = 0;
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            //当日时间段平均光伏出力>=第一口井运行负荷时,该时间为第一口井运行时间
                            if (predictedPower.getPower().compareTo(serviceRating) >= BusinessConstant.ZERO) {
                                startIndex = a;
                                break;
                            }
                        }
                        DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                        DateTime startTimeOptimize = DateUtil.date(start.getCreateTime());
                        //计算第一次关井时间,按照间开时间段顺延
                        int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                        DateTime endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, startDuration);
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ZERO, optimizeDate
                        );
                        firstStartTime = startTimeOptimize;
                        //计算时间偏移,取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(startTimeOptimize, startTime);
                    }
                    //其它井口的第一次启动时间
                    else {
                        DateTime startTimeOffset = firstStartTime.offsetNew(DateField.MINUTE, startInterval);
                        int startIndex = -1;
                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                            if (DateUtil.date(predictedPower.getCreateTime()).compareTo(startTimeOffset) >= 0) {
                                //判断第一口井启动时间+启动间隔时日平均光伏出力-前两口井的运行功率是否为正数
                                if (predictedPower.getPower().compareTo(totalOperatingPower) >= 0) {
                                    //确定第二口井第一次开井时间为第一口井启动时间+启动间隔
                                    startIndex = a;
                                    break;
                                }
                            }
                        }
                        if (startIndex > -1) {
                            DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                            DateTime startTimeOptimize = DateUtil.date(start.getCreateTime());
                            //计算未优化启动间隔
                            int openDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                            DateTime endTimeOptimize = startTimeOptimize.offsetNew(DateField.MINUTE, openDuration);
                            this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                    startTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT), BaseUtils.getEndTimeString(endTimeOptimize),
                                    BusinessConstant.ZERO, optimizeDate
                            );
                            //取时间间隔(分钟)
                            between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                            //修改启动时间
                            firstStartTime = startTimeOptimize;
                        } else {
                            //无法优化
                        }
                    }
                } else {
                    DateTime startOffset = DateUtil.offsetMinute(startTime, between);
                    DateTime endOffset = DateUtil.offsetMinute(endTime, between);
                    if (startOffset.compareTo(BusinessConstant.DATE_FLAG) < 0 && endOffset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                        //如果时间超过当天,舍弃
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                BusinessConstant.START_OF_DAY_TIME,
                                endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else if (endOffset.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else {
                        //计算偏移
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    }
                }
            }
        }
    }

    /**
     * 消峰平谷策略
     *
     * @param context                上下文
     * @param wellheadDTOList        井口dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param durationDTOList        持续时间dtolist
     * @param wellheadViewList       井口视图列表
     * @param durationMap            持续时间图
     * @param detail                 细节
     * @param monthNum               月份
     * @param periodId               id
     * @param optimizeDate           优化日期
     */
    private void peakEliminationAndValleyLevelingStrategy(XContext context, List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                                          List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceInstitutionWellheadView> wellheadViewList,
                                                          Map<String, List<SpaceInstitutionDurationEnt>> durationMap, SpaceInstitutionDetailEnt detail,
                                                          int monthNum, String periodId, DateTime optimizeDate) {
        //判断是否第一口井
        boolean isFirstWellhead;
        //时间差
        int between = 0;
        //启动间隔累积
        int startInterval = 0;
        //第一口井启动时间
        DateTime firstStartTime = null;
        //第一口井结束时间
        DateTime firstEndTime = null;
        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
            String wellheadId = wellhead.getWellheadId();
            String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellheadId);
            if (CollUtil.isEmpty(durationConfigList)) {
                //没有设置时间段,无法优化
                continue;
            }
            //保存原始记录
            for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
            }
            if (w == 0) {
                //第一个井口
                isFirstWellhead = true;
            } else {
                //计算启动间隔
                startInterval = startInterval + detail.getStartInterval();
                isFirstWellhead = false;
            }
            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                DateTime startTime = DateUtil.date(duration.getOpenTime());
                Date closeTime = duration.getCloseTime();
                DateTime endTime;
                if (ObjectUtil.isNull(closeTime)) {
                    endTime = DateUtil.beginOfDay(DateUtil.tomorrow());
                } else {
                    endTime = DateUtil.date(closeTime);
                }
                //第一次启动
                if (d == 0) {
                    //第一口井的启动时间
                    if (isFirstWellhead) {
                        String startTimeString;
                        //通过线路ID和月份获取市电峰谷策略明细配置
                        List<GetBasePriceStrategyDetailOutput> strategyDetailList = ServiceUtil.getStrategyDetailList(context,
                                GetBasePriceStrategyDetailInput.builder()
                                        .lineId(detail.getLineId())
                                        .strategyMonth(String.valueOf(monthNum))
                                        .build()
                        );
                        if (CollUtil.isNotEmpty(strategyDetailList)) {
                            //获取第一段谷电阶段的开始时间为第一口井的开井时间
                            Optional<GetBasePriceStrategyDetailOutput> low = strategyDetailList.stream()
                                    .filter(s -> !StringUtils.isAnyBlank(s.getEndTime(), s.getStartTime()) && StringUtils.equals("LOW", s.getPeriodTypeKey()))
                                    .findFirst();
                            if (low.isPresent()) {
                                GetBasePriceStrategyDetailOutput strategyDetailOutput = low.get();
                                //第一口井启动时间
                                firstStartTime = DateUtil.parse(strategyDetailOutput.getStartTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                firstEndTime = DateUtil.parse(strategyDetailOutput.getEndTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                startTimeString = strategyDetailOutput.getStartTime();
                            } else {
                                //没有谷电,不优化
                                firstStartTime = startTime;
                                firstEndTime = endTime;
                                startTimeString = duration.getOpenWellTime();
                            }
                        } else {
                            //没有配置,不优化
                            firstStartTime = startTime;
                            firstEndTime = endTime;
                            startTimeString = duration.getOpenWellTime();
                        }
                        //计算优化后的时间差优化后的第一口井开井时间-优化前第一次开井时间
                        int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                        //第一次关井时间按照启动时长顺延
                        DateTime endTimeOptimize = firstStartTime.offsetNew(DateField.MINUTE, startDuration);
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startTimeString, endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ZERO, optimizeDate
                        );
                        //取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                    }//其它井口的第一次启动时间
                    else {
                        //其它井启动时间:第一口井启动时间+启动间隔
                        DateTime startTimeOffset = firstStartTime.offsetNew(DateField.MINUTE, startInterval);
                        DateTime endTimeOffset = firstEndTime.offsetNew(DateField.MINUTE, startInterval);
                        //取时间间隔(分钟)
                        between = BaseUtils.getTimeDifferenceMinute(endTimeOffset, endTime);
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startTimeOffset.toString(BusinessConstant.MINUTES_FORMAT), endTimeOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                        //修改启动时间
                        firstStartTime = startTimeOffset;
                    }
                } else {
                    DateTime startOffset = DateUtil.offsetMinute(startTime, between);
                    DateTime endOffset = DateUtil.offsetMinute(endTime, between);
                    if (startOffset.compareTo(BusinessConstant.DATE_FLAG) < 0 && endOffset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                        //如果时间超过当天,舍弃
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                BusinessConstant.START_OF_DAY_TIME,
                                endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else if (endOffset.compareTo(BusinessConstant.DATE_FLAG) == 0) {
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.END_OF_DAY_TIME,
                                BusinessConstant.ONE, optimizeDate
                        );
                    } else {
                        //计算偏移
                        this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                startOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                endOffset.toString(BusinessConstant.MINUTES_FORMAT),
                                BusinessConstant.ONE, optimizeDate
                        );
                    }
                }
            }
        }
    }

    /**
     * 保存不需要优化的井口
     *
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param wellheadDTOList        井口dtolist
     * @param unOptimizeWellheadList 取消优化井口列表
     * @param durationMap            持续时间图
     * @param periodId               期间id
     * @param optimizeDate           优化日期
     */
    public void setUnOptimizeWellheadConfig(List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                            List<SpaceOptimizeWellheadDTO> wellheadDTOList, List<SpaceInstitutionWellheadView> unOptimizeWellheadList,
                                            Map<String, List<SpaceInstitutionDurationEnt>> durationMap, String periodId, DateTime optimizeDate) {
        if (CollUtil.isNotEmpty(unOptimizeWellheadList)) {
            for (SpaceInstitutionWellheadView wellhead : unOptimizeWellheadList) {
                String wellheadId = wellhead.getWellheadId();
                String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber(), wellhead.getStartSeq(), optimizeDate, BusinessConstant.ONE);
                switch (wellhead.getIntervalTypeKey()) {
                    //小间开(不优化)
                    case "0":
                        if (!durationMap.containsKey(wellheadId)) {
                            continue;
                        }
                        //初始化未优化间开
                        List<SpaceInstitutionDurationEnt> durationList = durationMap.get(wellheadId);
                        for (SpaceInstitutionDurationEnt durationEnt : durationList) {
                            this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId, optimizeDate);
                            this.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                    durationEnt.getOpenWellTime(), durationEnt.getCloseWellTime(),
                                    BusinessConstant.ONE, optimizeDate
                            );
                        }
                        break;
                    //大间开
                    case "1":
                        //创建间开记录(全开)
                        this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                BusinessConstant.ONE, optimizeDate);
                        this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                BusinessConstant.ONE, optimizeDate);
                        break;
                    default:
                        if (StringUtils.equals(wellhead.getRunTypeKey(), BusinessConstant.CONTINUOUS_PUMPING_WELL)) {
                            this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                            this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.END_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                        } else {
                            //未配置全关
                            this.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.START_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                            this.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, BusinessConstant.START_OF_DAY_TIME, null,
                                    BusinessConstant.ONE, optimizeDate);
                        }
                }
            }
        }
    }

    /**
     * 获得最大功率
     *
     * @param avgPowerList 平均功率列表
     * @return {@link BigDecimal}
     */
    private BigDecimal getPowerMax(List<DynamicQueryPlantPredictedPowerOutput> avgPowerList) {
        return avgPowerList.stream()
                .map(DynamicQueryPlantPredictedPowerOutput::getPower)
                .max(BigDecimal::compareTo)
                .orElse(BigDecimal.ZERO);
    }

    /**
     * 获取井口总功率
     *
     * @param spaceWellheadList 空间井口清单
     * @param detailId          详细信息id
     * @return {@link BigDecimal}
     */
    private BigDecimal getWellheadTotalPower(List<SpaceInstitutionWellheadView> spaceWellheadList, String detailId) {
        return spaceWellheadList.stream()
                .filter(w -> StringUtils.equals(detailId, w.getInstitutionId()))
                .map(SpaceInstitutionWellheadView::getServiceRating)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * 获取当前制度关联的井口按照是否可优化分组
     *
     * @param spaceWellheadList 空间井口清单
     * @param detailId          详细信息id
     * @return {@link Map}<{@link Boolean}, {@link List}<{@link SpaceInstitutionWellheadView}>>
     */
    private Map<Boolean, List<SpaceInstitutionWellheadView>> getWellheadViewList(List<SpaceInstitutionWellheadView> spaceWellheadList, String detailId) {
        return spaceWellheadList.stream()
                .sorted(Comparator.comparing(SpaceInstitutionWellheadView::getStartSeq))
                .collect(
                        Collectors.partitioningBy(w -> StringUtils.equals(detailId, w.getInstitutionId()) &&
                                StringUtils.equals(BusinessConstant.INTERVAL_PUMPING_WELL, w.getRunTypeKey()) &&
                                StringUtils.equals("0", w.getIntervalTypeKey())
                        )
                );
    }

    /**
     * 获取井口间开时间段
     *
     * @param context  上下文
     * @param detailId 详细信息id
     * @return {@link Map}<{@link String}, {@link List}<{@link SpaceInstitutionDurationEnt}>>
     */
    private Map<String, List<SpaceInstitutionDurationEnt>> getDurationMap(XContext context, String detailId) {
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        //通过间开ID和井口ID查所有井口时段配置
        List<SpaceInstitutionDurationEnt> durationList = durationMapper.selectList(new QueryWrapper<SpaceInstitutionDurationEnt>()
                .select("*",
                        "STR_TO_DATE( CONCAT( open_well_time, ':00' ), '%H:%i:%s' ) AS openTime",
                        "STR_TO_DATE( CONCAT( close_well_time, ':00' ), '%H:%i:%s' ) AS closeTime"
                )
                .lambda()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDurationEnt::getInstitutionId, detailId)
                .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
        );
        if (CollUtil.isEmpty(durationList)) {
            //没有设置时间段,无法优化
            return new HashMap<>(0);
        }
        return durationList.stream()
                .collect(
                        Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId)
                );
    }

    /**
     * 按线路ID集合获取电站列表
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    private List<DynamicQueryBasePowerLinePlantOutput> getPowerLinePlantListByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantOutput> result = cloudService.queryPowerLinePlantListByParam(context,
                DynamicQueryBasePowerLinePlantInput.builder()
                        .lineIds(lineIds)
                        .build()
        );
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 条件查询线路关联储能设备参数平均值
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link Map}<{@link String}, {@link List}<{@link DynamicQueryBasePowerLineStorageViewOutput}>>
     */
    private Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> getPowerLineStorageAvgByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineStorageViewOutput> result = cloudService.queryPowerLineStorageAvgByLineIds(context,
                DynamicQueryBasePowerLineStorageInput.builder()
                        .lineIds(lineIds)
                        .build()
        );
        result.throwIfFail();
        List<DynamicQueryBasePowerLineStorageViewOutput> list = result.getResult();
        Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> collect;
        if (CollUtil.isEmpty(list)) {
            collect = new HashMap<>(0);
        } else {
            collect = list.stream()
                    .collect(Collectors.groupingBy(DynamicQueryBasePowerLineStorageViewOutput::getLineId));
        }
        return collect;
    }

    /**
     * 查询线路下所有储能设备ID
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link Map}<{@link String}, {@link List}<{@link String}>>
     */
    private Map<String, List<String>> getPowerLineStorageListByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineStorageOutput> result = cloudService.queryPowerLineStorageListByLineIds(context,
                DynamicQueryBasePowerLineStorageInput.builder()
                        .lineIds(lineIds)
                        .build());
        result.throwIfFail();
        List<DynamicQueryBasePowerLineStorageOutput> list = result.getResult();
        Map<String, List<String>> collect;
        if (CollUtil.isEmpty(list)) {
            collect = new HashMap<>(0);
        } else {
            collect = list.stream()
                    .collect(
                            Collectors.groupingBy(DynamicQueryBasePowerLineStorageOutput::getLineId,
                                    Collectors.mapping(DynamicQueryBasePowerLineStorageOutput::getStorageId, Collectors.toList()))
                    );
        }
        return collect;
    }

    /**
     * 为间开井口列表设置发电功率
     *
     * @param context           上下文
     * @param spaceWellheadList 空间井口清单
     */
    private void setServiceRatingForSpaceWellheadList(XContext context, List<SpaceInstitutionWellheadView> spaceWellheadList) {
        IBaseWellheadCloudService cloudService = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = cloudService.queryBaseWellheadListByParam(context,
                DynamicQueryBaseWellheadInput.builder()
                        .wellheadIds(spaceWellheadList.stream()
                                .map(SpaceInstitutionWellheadView::getWellheadId)
                                .collect(Collectors.toList())
                        )
                        .build());
        result.throwIfFail();
        List<DynamicQueryBaseWellheadOutput> wellheadOutputList = result.getResult();
        if (CollUtil.isEmpty(wellheadOutputList)) {
            return;
        }
        spaceWellheadList.forEach(s -> {
            s.setServiceRating(
                    wellheadOutputList.stream()
                            .filter(w -> StringUtils.equals(s.getWellheadId(), w.getId()))
                            .findAny()
                            .map(DynamicQueryBaseWellheadOutput::getServiceRating)
                            .orElse(BigDecimal.ZERO)
            );
        });
    }

    /**
     * 条件查询时段储能电量平均值(模拟测试用)
     * todo: 模拟测试用,后续替换
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryStoragePredictedPowerOutput}>
     */
    private List<DynamicQueryStoragePredictedPowerOutput> getAverageEnergyStorageListByParam(XContext context, DynamicQueryStoragePredictedPowerInput input) {
        IStoragePredictedPowerCloudService cloudService = context.getBean(IStoragePredictedPowerCloudService.class);
        XListResult<DynamicQueryStoragePredictedPowerOutput> result = cloudService.queryAverageEnergyStorageListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取井口Map通过最大功率区分
     *
     * @param wellheadViewList 井口视图列表
     * @param powerMax         最大功率
     * @return {@link Map}<{@link Boolean}, {@link List}<{@link SpaceInstitutionWellheadView}>>
     */
    private Map<Boolean, List<SpaceInstitutionWellheadView>> getWellheadViewMapByPower(List<SpaceInstitutionWellheadView> wellheadViewList, BigDecimal powerMax) {
        return wellheadViewList.stream()
                .collect(
                        Collectors.partitioningBy(w -> powerMax.compareTo(w.getServiceRating()) >= BusinessConstant.ZERO)
                );
    }

    /**
     * 离网型间开记录
     *
     * @param offGridPeriodList 离网时段列表
     * @param wellheadId        井口id
     * @param openTime          断开时间
     * @param closeTime         关闭时间
     * @param firstStart        是否第一次启动
     */
    private void createDurationOffGridPeriod(List<SpaceOptimizeDurationDTO> offGridPeriodList, String wellheadId,
                                             Date openTime, Date closeTime, Integer firstStart) {
        SpaceOptimizeDurationDTO durationDTO = new SpaceOptimizeDurationDTO();
        durationDTO.setWellheadId(wellheadId);
        durationDTO.setOpenTime(openTime);
        durationDTO.setCloseTime(closeTime);
        durationDTO.setFirstStart(firstStart);
        offGridPeriodList.add(durationDTO);
    }

    /**
     * 是否在优化
     *
     * @param context  上下文
     * @param detailId 详细信息id
     * @param flag     旗帜
     * @param now      现在
     * @return boolean
     */
    private boolean isOptimize(XContext context, String detailId, Integer flag, DateTime now) {
        switch (flag) {
            case 0:
                SpaceOptimizeLongPeriodMapper periodMapper = context.getBean(SpaceOptimizeLongPeriodMapper.class);
                SpaceOptimizeLongPeriodEnt longPeriodEnt = periodMapper.selectOne(new QueryWrapper<SpaceOptimizeLongPeriodEnt>()
                        .select("institution_id",
                                "MAX( optimize_deadline ) AS optimize_deadline")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeLongPeriodEnt::getInstitutionId, detailId)
                );
                if (ObjectUtil.isNotNull(longPeriodEnt)) {
                    Date optimizeDead = longPeriodEnt.getOptimizeDeadline();
                    if (DateUtil.compare(now, optimizeDead) == -1) {
                        return true;
                    }
                }
                return false;
            case 1:
                SpaceOptimizeMidPeriodMapper midPeriodMapper = context.getBean(SpaceOptimizeMidPeriodMapper.class);
                SpaceOptimizeMidPeriodEnt midPeriodEnt = midPeriodMapper.selectOne(new QueryWrapper<SpaceOptimizeMidPeriodEnt>()
                        .select("institution_id",
                                "MAX( optimize_deadline ) AS optimize_deadline")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeMidPeriodEnt::getInstitutionId, detailId)
                );
                if (ObjectUtil.isNotNull(midPeriodEnt)) {
                    Date optimizeDead = midPeriodEnt.getOptimizeDeadline();
                    if (DateUtil.compare(now, optimizeDead) == -1) {
                        return true;
                    }
                }
                return false;
            case 2:
                SpaceOptimizeUltraPeriodMapper ultraPeriodMapper = context.getBean(SpaceOptimizeUltraPeriodMapper.class);
                SpaceOptimizeUltraPeriodEnt ultraPeriodEnt = ultraPeriodMapper.selectOne(new QueryWrapper<SpaceOptimizeUltraPeriodEnt>()
                        .select("institution_id",
                                "MAX( optimize_deadline ) AS optimize_deadline")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeUltraPeriodEnt::getInstitutionId, detailId)
                );
                if (ObjectUtil.isNotNull(ultraPeriodEnt)) {
                    Date optimizeDead = ultraPeriodEnt.getOptimizeDeadline();
                    if (DateUtil.compare(now, optimizeDead) == -1) {
                        return true;
                    }
                }
            default:
                return false;
        }
    }
}