package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.prediction.entity.EnergyConsumptionAnalysisEnt;
import pps.core.prediction.entity.ThirdDailyAccumulationUpdateEnt;
import pps.core.prediction.enums.BusinessError;
import pps.core.prediction.mapper.ThirdDailyAccumulationUpdateMapper;
import pps.core.prediction.service.data.energy_consumption_analysis.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 能耗分析模块
 *
 * @author ZWT
 * @date 2023/11/23
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService {

    /**
     * 能耗分析--能耗概览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("能耗分析--能耗概览")
    @XApiGet
    public XSingleResult<GetEnergyConsumptionAnalysisViewOutput> getEnergyConsumptionAnalysisView(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        GetEnergyConsumptionAnalysisViewOutput output = new GetEnergyConsumptionAnalysisViewOutput();
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isEmpty(lineList)) {
            output.setPowerGeneration(BigDecimal.ZERO);
            output.setPowerConsumption(BigDecimal.ZERO);
            output.setAbsorptionRate(BigDecimal.ZERO);
        } else {
            List<String> lineNameList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            input.getBetweenDate();
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                            "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                            "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption")
                    .lambda()
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNameList)
                    .between(ThirdDailyAccumulationUpdateEnt::getCreateDate, input.getStartTime(), input.getEndTime())
            );
            output.setPowerGeneration(analysisEnt.getPhotovoltaicPower().setScale(2, RoundingMode.HALF_UP));
            output.setPowerConsumption(analysisEnt.getDailyElectricityConsumption().setScale(2, RoundingMode.HALF_UP));
            output.setAbsorptionRate(this.getAbsorptionRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption()));
        }
        return XSingleResult.success(output);
    }

    /**
     * 能耗分析--绿电消纳/用电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("能耗分析--绿电消纳/用电趋势")
    @XApiGet
    public XListResult<DynamicQueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisList(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        List<DynamicQueryEnergyConsumptionAnalysisViewOutput> list = new ArrayList<>(16);
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isNotEmpty(lineList)) {
            List<String> lineNameList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            input.getBetweenDate();
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            List<ThirdDailyAccumulationUpdateEnt> analysisEntList = analysisMapper.selectList(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                            "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                            "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption",
                            "DATE( create_date ) AS create_date")
                    .lambda()
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNameList)
                    .between(ThirdDailyAccumulationUpdateEnt::getCreateDate, input.getStartTime(), input.getEndTime())
                    .last("GROUP BY DATE( create_date ) ")
            );
            if (CollUtil.isNotEmpty(analysisEntList)) {
                for (ThirdDailyAccumulationUpdateEnt ent : analysisEntList) {
                    list.add(DynamicQueryEnergyConsumptionAnalysisViewOutput.builder()
                            .powerConsumption(ent.getDailyElectricityConsumption())
                            .powerGeneration(ent.getPhotovoltaicPower())
                            .dataDate(ent.getCreateDate())
                            .dateFormat(DateUtil.format(ent.getCreateDate(), "MM-dd"))
                            .absorptionRate(this.getAbsorptionRate(ent.getInPlaceConsumption(), ent.getDailyElectricityConsumption()))
                            .build());
                }
            }
        }
        return XListResult.success(list);
    }

    /**
     * 能耗分析--用电详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("能耗分析--用电详情")
    @XApiGet
    public XPageResult<QueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisView(XContext context, QueryEnergyConsumptionAnalysisPageInput input) {
        List<QueryEnergyConsumptionAnalysisViewOutput> outputs = new ArrayList<>(input.getLimit());
        List<ThirdDailyAccumulationUpdateEnt> list;
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isNotEmpty(lineList)) {
            List<String> lineNameList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            PageHelper.startPage(input.getPage(), input.getLimit());
            input.getBetweenDate();
            ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            Date startTime = input.getStartTime();
            Date endTime = input.getEndTime();
            //查本期数据
            list = mapper.selectList(this.getQueryWrapper4Page(lineNameList, startTime, endTime));
            if (CollUtil.isNotEmpty(list)) {
                PageHelper.clearPage();
                //查上期数据
                switch (input.getDateType()) {
                    case 1:
                        endTime = DateUtil.offsetDay(startTime, -1);
                        startTime = DateUtil.offsetDay(startTime, -7);
                        break;
                    case 2:
                        endTime = DateUtil.offsetDay(startTime, -1);
                        startTime = DateUtil.offsetDay(startTime, -30);
                        break;
                    case 3:
                        int betweenDay = (int) DateUtil.betweenDay(startTime, endTime, true) + 1;
                        endTime = DateUtil.offsetDay(startTime, -1);
                        startTime = DateUtil.offsetDay(startTime, -betweenDay);
                        break;
                    default:
                        throw new XServiceException(BusinessError.DateTypeError);
                }
                List<ThirdDailyAccumulationUpdateEnt> periodList = mapper.selectList(this.getQueryWrapper4Page(lineNameList, startTime, endTime));
                if (CollUtil.isNotEmpty(periodList)) {
                    for (ThirdDailyAccumulationUpdateEnt ent : list) {
                        EnergyConsumptionAnalysisEnt period;
                        ThirdDailyAccumulationUpdateEnt one = CollUtil.findOne(periodList, x -> CharSequenceUtil.equals(ent.getStationName(), x.getStationName()));
                        if (Objects.isNull(one)) {
                            period = EnergyConsumptionAnalysisEnt.builder()
                                    .powerConsumption(BigDecimal.ZERO)
                                    .powerGeneration(BigDecimal.ZERO)
                                    .build();
                        } else {
                            period = EnergyConsumptionAnalysisEnt.builder()
                                    .powerConsumption(one.getDailyElectricityConsumption())
                                    .powerGeneration(one.getPhotovoltaicPower())
                                    .build();
                        }
                        outputs.add(QueryEnergyConsumptionAnalysisViewOutput.builder()
                                .lineName(ent.getStationName())
                                .powerGeneration(ent.getPhotovoltaicPower())
                                .generationCycleRatio(this.getCycleRatio(ent.getPhotovoltaicPower(), period.getPowerGeneration()))
                                .powerConsumption(ent.getDailyElectricityConsumption())
                                .consumptionCycleRatio(this.getCycleRatio(ent.getDailyElectricityConsumption(), period.getPowerConsumption()))
                                .absorptionRate(this.getAbsorptionRate(ent.getInPlaceConsumption(), ent.getDailyElectricityConsumption()))
                                .build());
                    }
                } else {
                    for (ThirdDailyAccumulationUpdateEnt ent : list) {
                        outputs.add(QueryEnergyConsumptionAnalysisViewOutput.builder()
                                .lineName(ent.getStationName())
                                .powerGeneration(ent.getPhotovoltaicPower())
                                .generationCycleRatio(BigDecimal.ZERO)
                                .powerConsumption(ent.getDailyElectricityConsumption())
                                .consumptionCycleRatio(BigDecimal.ZERO)
                                .absorptionRate(this.getAbsorptionRate(ent.getInPlaceConsumption(), ent.getDailyElectricityConsumption()))
                                .build());
                    }
                }
            }
        } else {
            list = new ArrayList<>(0);
        }
        PageInfo<ThirdDailyAccumulationUpdateEnt> pageInfo = new PageInfo<>(list);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 分页功能封装查询条件
     *
     * @param lineNames 线路名称
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return {@link LambdaQueryWrapper}<{@link EnergyConsumptionAnalysisEnt}>
     */
    private LambdaQueryWrapper<ThirdDailyAccumulationUpdateEnt> getQueryWrapper4Page(Collection<String> lineNames, Date startTime, Date endTime) {
        return new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                        "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                        "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption",
                        "station_name")
                .lambda()
                .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNames)
                .between(ThirdDailyAccumulationUpdateEnt::getCreateDate, startTime, endTime)
                .groupBy(ThirdDailyAccumulationUpdateEnt::getStationName)
                .orderByAsc(ThirdDailyAccumulationUpdateEnt::getPhotovoltaicPower);
    }

    /**
     * 获取消纳率
     * 消纳率:就地消纳量/累计用电*100%
     *
     * @param powerGeneration  发电
     * @param powerConsumption 功耗
     * @return {@link BigDecimal}
     */
    private BigDecimal getAbsorptionRate(BigDecimal powerGeneration, BigDecimal powerConsumption) {
        BigDecimal absorptionRate;
        if (powerConsumption.compareTo(BigDecimal.ZERO) > 0) {
            absorptionRate = powerGeneration.divide(powerConsumption, 4, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal(100));
        } else {
            absorptionRate = BigDecimal.ZERO;
        }
        return absorptionRate;
    }

    /**
     * 获取环比
     * (本期-上期) /上期*100%
     *
     * @param currentPeriod 本期
     * @param lastPeriod    上期
     * @return {@link BigDecimal}
     */
    private BigDecimal getCycleRatio(BigDecimal currentPeriod, BigDecimal lastPeriod) {
        BigDecimal cycleRatio;
        if (lastPeriod.compareTo(BigDecimal.ZERO) > 0) {
            cycleRatio = currentPeriod.subtract(lastPeriod)
                    .divide(lastPeriod, 4, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal(100));
        } else {
            cycleRatio = BigDecimal.ZERO;
        }
        return cycleRatio;
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    private List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 查询组织列表下所有线路
     *
     * @param context  上下文
     * @param ouIdList ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, List<String> ouIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineListByOuIdList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIdList)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }
}
